package SendMail;
use Fcntl qw(:flock);
use strict;
{ use bytes; }
use utf8;
######################################################################
# メールアドレスを送信するモジュール
# SendMail.pm 1.3.2
# Copyright (C) 1999-2015 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 
# 注意：Jcode.pm[Copyright 1999-2005 Dan Kogai <dankogai@dan.co.jp>]が必要です。
# 最終更新日：2015-02-05
######################################################################
# 
# このサブルーチン集の使い方
# ---------------------------------------------------------------------------------------
#	(0) オブジェクトを作る。
#		require CMONOS::CGI_Lib::SendMail;
#		obj = SendMail->new(sendmail,cgi_master[,limit,input,macbinary,smtpserver]);
# 			sendmail		sendmailのパス
# 							SMTPホストネーム
# 								※ 何も指定が無い場合`/bin/hostname`
# 			cgi_master		管理人アドレス（cgi_masterを設定しないとメールは送信されません。）
# 			limit			添付ファイルのファイルサイズ上限（設定無しで無制限）
# 			input			アップロード情報を保持しているオブジェクト
# 			macbinary		1 = マックバイナリーを削除する
# 			smtpserver		SMTPサーバのアドレス
# 
#		inputオブションはCommon.pmと併用する際に使います。
#		アップロードしたファイルを一時保存せず直接添付したいときに便利です。
# 
#		例）
#			<input name="to" type="text" size="30" />
#			<input name="from" type="text" size="30" />
#			<input name="subject" type="text" size="30" />
#			<input name="message" type="text" size="30" />
#			<input name="file" type="file" />
# 
#				↓
# 
#			use Common;
#			$input = Common->new;
#			$input->decode;
#			use SendMail;
#			$mail = SendMail->new('/usr/sbin/sendmail','cgimaster@dokosoko.com',$input);
#			obj->send_mail(
#				'To'			=>  $to,
#				'From'		=>  $from,
#				'Subject'	=>  $subject,
#				'message'	=>  $message,			#messageは本文
#				'file'		=>  '<file>',			#fileは添付するファイルのシンボリックネーム
#			);
# 
# ---------------------------------------------------------------------------------------
#	(1) メール送信
#		obj->send_mail(
#			'To'			=>  'name <name@dokosoko.com>',
#			'Cc'			=>  'name2 <name2@dokosoko.com>,name3 <name3@dokosoko.com>',
#			'Bcc'			=>  'name4 <name4@dokosoko.com>',
#			'Bcc'			=>  'name5 <name5@dokosoko.com>',
#			'From'		=>  'owner <owner@dokosoko.com>',
#			'Subject'	=>  '題名',
#			'RCPT TO'	=>  \@addr,				#送信先アドレスのリファレンス
#			'message'	=>  'message',			#messageは本文
#			'file'		=>  'path1,path2',		#fileは添付するファイルのパス
#			'file'		=>  '<name>',			#fileは添付するファイルのシンボリックネーム
#			'file'		=>  \%data,				#$data{'name'}=添付ファイルファイル名
#												#$data{'type'}=MIME-TYPE
#												#$data{'data'}=データ
#			'file'		=>  \@files,			#添付ファイルデータ配列のリファレンス
# 			'input'		=>  $input,				#アップロード情報を保持しているオブジェクト
# 			'macbinary'	=>  '1',				#1 = マックバイナリーを削除する
#			'unlink'	=>  '1',				#file送信後ファイルを削除するかどうか(1=削除)
#			'encoding'	=>  'sjis',				#メッセージの文字コード（utf8/euc/jis/sjis）
#			'info'		=>  '1'					#メッセージに送信情報を付記するか（1=付記）
#		);
# 
#		※オブジェクト作成時にcgi_masterを設定しないとメールは送信されません。
#		※To.Cc.Bccを合わせた宛先の総数は100件以内にしてください。
#		　100件目以降の宛先は無視されます。
#		※送信に失敗したときは、添付ファイルを削除しません。
# 
#	(1)-0 ソケットを開いて接続する
#		obj->tcp_connect(HANDLE,host,port,nobuf);
# 			HANDLE		メール送信用ファイルハンドル
# 			host			IPアドレス or ホスト名
# 			port			アプリケーションプロトコル名
# 			nobuf			1=バッファリングを止める
# 
# ---------------------------------------------------------------------------------------
#	(2) メールヘッダBASE64エンコード
#		obj->header_to_base64(header);
# 			メールヘッダの日本語部分のみBASE64エンコードします。
#		obj->text_to_base64(text,op);
# 			テキストをBASE64エンコードします。
# 			op=1にすると、テキストが既にJISであると判断し、変換を行いません。
# 
# ---------------------------------------------------------------------------------------
#	(3) BASE64エンコード
#		obj->data_to_base64(data,fold);
# 			fold			1=72文字で折り返す。
# 
# ---------------------------------------------------------------------------------------
#	(4)-1 @msg = obj->dialog;
#		SendMail.pmの処理で発生したエラーのエラーメッセージを返します。
#		JavaScriptのalertを意識し改行は'\n'となっています。
# 
#	(4)-1 @bad_address = obj->bad_address;
#		送信に失敗したメールアドレスのリストを返します（sendmail経由では機能しません）。
# 
# 
######################################################################
#初期設定
BEGIN {

	#拡張子とMIME-TYPEの対応
	#設定に無い場合"application/octet-stream"と送信します。
	$SendMail::mime_type = (%Common::mime_type) ? \%Common::mime_type : {
		'txt' => 'text/plain',
		'html' => 'text/html',
		'htm' => 'text/html',
		'gif' => 'image/gif',
		'jpg' => 'image/jpeg',
		'png' => 'image/png',
		'mid' => 'audio/midi',
		'mpg' => 'video/mpeg',
		'mov' => 'video/quicktime',
		'avi' => 'video/x-msvideo',
		'swf' => 'application/x-shockwave-flash',
		'dcr' => 'application/x-director',
		'csv' => 'text/comma-separated-values',
		'tsv' => 'text/tab-separated-values',
		'pdf' => 'application/pdf',
		'hqx' => 'application/mac-binhex40',
		'lzh' => 'application/lha',
		'zip' => 'application/zip',
		'bin' => 'application/x-macbinary',
		'sit' => 'application/x-stuffit',
		'ra' => 'audio/x-pn-realaudio',
		'mp3' => 'audio/mpeg',
		'm4a' => 'audio/m4a',
		'pct' => 'image/pict',
		'pict' => 'image/pict',
		'tif' => 'image/tiff',
		'ico' => 'image/x-icon',
		'mp4' => 'video/mp4',
		'flv' => 'video/x-flv',
		'wmv' => 'video/x-ms-wmv',
		'asf' => 'video/x-ms-asf',
		'aac' => 'audio/aac',
		'm4v' => 'video/mp4',
		'ogg' => 'video/ogg',
		'ogv' => 'video/ogg',
		'oga' => 'audio/ogg',
		'webm' => 'video/webm',
		'3g2' => 'video/3gpp2',
		'xml' => 'text/xml',
		'kml' => 'text/xml',
		'epub' => 'application/epub+zip',
		'eps' => 'application/postscript',
		'ai' => 'application/postscript',
		'psd' => 'application/vnd.adobe.photoshop',
		'tex' => 'application/x-tex',
		'tgz' => 'application/x-compressed',
		'wav' => 'audio/wav',
		'ppt' => 'application/vnd.ms-powerpoint',
		'xls' => 'application/vnd.ms-excel',
		'doc' => 'application/msword',
		'pptx' => 'application/vnd.openxmlformats-officedocument.presentationml.presentation',
		'xlsx' => 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet',
		'docx' => 'application/vnd.openxmlformats-officedocument.wordprocessingml.document',
		'odt' => 'application/vnd.oasis.opendocument.text',
		'ods' => 'application/vnd.oasis.opendocument.spreadsheet',
		'odp' => 'application/vnd.oasis.opendocument.presentation',
		'odg' => 'application/vnd.oasis.opendocument.graphics',
		'odc' => 'application/vnd.oasis.opendocument.chart',
		'odf' => 'application/vnd.oasis.opendocument.formula',
		'odi' => 'application/vnd.oasis.opendocument.image',
		'odb' => 'application/vnd.oasis.opendocument.database',
	};
}

######################################################################
#初期設定は以上です。
######################################################################
#オブジェクト作成
if (!ref $cgi::Jcode) {
	use Jcode;
	$cgi::Jcode = Jcode->new;
}

sub new ($$$;$$$) {
my($pkg,$path,$cgi_master,$limit,$input,$macbinary,$host) = @_;
my $this = {};
$this->{'sendmail'} = (defined $path && $path) ? $path : '';
$this->{'cgi_master'} = $cgi_master if defined $cgi_master && $cgi_master ne '';
$this->{'limit'} = $limit if defined $limit && $limit;
$this->{'input'} = $input if defined $input && $input ne '';
$this->{'macbinary'} = 1 if defined $macbinary && $macbinary;
chomp($host = `/bin/hostname`) if $host && $host eq 'auto';
$this->{'host'} = (defined $host && $host) ? $host : '';
$this->{'CR+LF'} = ($this->{'sendmail'}) ? "\x0A" : "\x0D\x0A";
bless $this;
$this;
}

######################################################################
#メール送信
sub send_mail ($@) {
my $this = shift;
my($my_url,%header,$rcpt_to,$message,$info,$file,$unlink,$open,@files,$boundary,$client_ip);
local(*MAIL);
if (!defined $this->{'cgi_master'} || $this->{'cgi_master'} eq '') {
	push(@{$this->{'dialog'}},'メールの送信に失敗しました。管理人アドレスが設定されていません。');
	return 0;
}
$open = 0;
@files = ();
$this->{'encoding'} = 'utf8';

#デフォルトで送信情報を付加
$info = 1;

#SMTPヘッダを取得
while (my($k,$v) = splice(@_,0,2)) {
	next if $k eq '' || !defined $v || $v eq '';
	next if $k =~ /^(Date|Message-ID|Content|Mime-version|Received|In-Reply-To|UIDL|Status|Sender|X-Mailer)$/i;
	next if $k =~ /^Content-/i;
	if ($k eq 'message') {
		$message .= $v;
	} elsif ($k =~ /^file[0-9]*$/) {
		if (ref $v eq 'ARRAY') {
			push(@files,@$v);
		} elsif (ref $v) {
			push(@files,$v);
		} elsif ($v ne '') {
			$file .= ',' if defined $file;
			$file .= $v;
		}
	} elsif ($k eq 'unlink' && $v == 1) {
		$unlink = $v;
	} elsif ($k eq 'info' && $v == 0) {
		$info = 0;
	} elsif ($k eq 'encoding') {
		$this->{'encoding'} = $v;
	} elsif ($k eq 'input') {
		$this->{'input'} = $v;
	} elsif ($k eq 'macbinary') {
		$this->{'macbinary'} = $v;
	} elsif ($k eq 'RCPT TO') {
		$rcpt_to = $v if ref $v eq 'ARRAY';
	} else {
		$k =~ s/([0-9a-zA-Z_]+)/ucfirst(lc $1)/eg;
		$v =~ tr/\x0A\x0D/ /s;
		$header{$k} .= ', ' if defined $header{$k};
		$header{$k} .= $v;
	}
}

if (defined $file && $file =~ /<.*>/ && !defined $this->{'input'}) {
	push(@{$this->{'dialog'}},'スクリプトエラー！');
	return 0;
}

#if ($header{'To'} && $header{'To'} =~ /^([^,]+),(.*)$/s) {
#	$header{'To'} = $1;
#	$header{'Cc'} = (($header{'Cc'}) ?$header{'Cc'}. ',' : '') . $2;
#}

#アドレス指定ヘッダの書式チェック
if (defined $header{'To'} && $header{'To'} ne 'RCPT TO' && $header{'To'} ne '' && $this->is_invalid_email_address_header($header{'To'})) {
	push(@{$this->{'dialog'}},'宛先（To）の書式を確認してください。');
	$header{'To'} = '';
}
if (defined $header{'Cc'} && $header{'Cc'} ne 'RCPT TO' && $header{'Cc'} ne '' && $this->is_invalid_email_address_header($header{'Cc'})) {
	push(@{$this->{'dialog'}},'宛先（Cc）の書式を確認してください。');
	$header{'Cc'} = '';
}
if (defined $header{'Bcc'} && $header{'Bcc'} ne 'RCPT TO' && $header{'Bcc'} ne '' && $this->is_invalid_email_address_header($header{'Bcc'})) {
	push(@{$this->{'dialog'}},'宛先（Bcc）の書式を確認してください。');
	$header{'Bcc'} = '';
}
if (defined $header{'Reply-To'} && $header{'Reply-To'} ne 'RCPT TO' && $header{'Reply-To'} ne '' && $this->is_invalid_email_address_header($header{'Reply-To'})) {
	push(@{$this->{'dialog'}},'返信先（Reply-To）の書式を確認してください。');
	$header{'Reply-To'} = '';
}
if (defined $header{'From'} && $header{'From'} ne 'RCPT TO' && $header{'From'} ne '' && $this->is_invalid_email_address_header($header{'From'})) {
	push(@{$this->{'dialog'}},'差出人（From）の書式を確認してください。');
	$header{'From'} = '';
}
if ((!defined $header{'To'} || $header{'To'} eq '') && (!defined $header{'Cc'} || $header{'Cc'} eq '') && (!defined $header{'Bcc'} || $header{'Bcc'} eq '') && (!ref $rcpt_to || @$rcpt_to == 0)) {
	push(@{$this->{'dialog'}},'送信先がみつかりません。');
	return 0;
}

$my_url = $this->my_url;
$header{'X-Mailer'} = $my_url;
$header{'From'} = $this->{'cgi_master'} if !defined $header{'From'} || $header{'From'} eq '';
$header{'Reply-To'} = $header{'From'} if !defined $header{'Reply-To'} || $header{'Reply-To'} eq '';
$header{'Return-Path'} = $header{'Reply-To'} if !defined $header{'Return-Path'} || $header{'Return-Path'} eq '';
$header{'Subject'} = 'Message from ' . $my_url if !defined $header{'Subject'} || $header{'Subject'} eq '';
$header{'Date'} = $this->date;
$header{'Mime-Version'} = '1.0';

#送信情報の付加
if (defined $info && $info) {
	$message .= $this->{'CR+LF'};
	$message .= "※このメールはCGIから送信されました。$this->{'CR+LF'}";
	$message .= "------------------------------------------------------------$this->{'CR+LF'}";
	$message .= "送信情報$this->{'CR+LF'}";
	$message .= "URL: $my_url$this->{'CR+LF'}";
	$message .= "Date: $header{'Date'}$this->{'CR+LF'}";
	$message .= $this->{'CR+LF'};
	$message .= "送信者情報$this->{'CR+LF'}";
	$message .= 'IP: [' . ($ENV{'REMOTE_HOST'} || $this->to_host($ENV{'REMOTE_ADDR'})) . ']' . $this->{'CR+LF'};
	$message .= $this->{'CR+LF'};
	$message .= "問い合わせ$this->{'CR+LF'}";
	$message .= "E-Mail: $this->{'cgi_master'}$this->{'CR+LF'}";
	$message .= "------------------------------------------------------------$this->{'CR+LF'}";
}

#幅を揃える
#改行をCR+LFに変換
$message = $cgi::Jcode->set($this->mfold($this->jcode_fallback($message)),'utf8')->jis;
utf8::encode($message) if utf8::is_utf8($message);

#添付ファイル
push(@files,split(/,[\x09\x0A\x0C\x0D\x20]*/,$file)) if defined $file && $file ne '';

#sendmailを利用する
local(*MAIL);
if ($this->{'sendmail'}) {
	#sendmailに接続できないときは10秒ごとに10回再接続
	my($try);
	for ($try = 0; $try < 10; $try++) {
		if (open(MAIL, "|$this->{'sendmail'} -oi -t")) {
			binmode MAIL;
			$open = 1;
			last;
		}
		sleep(10);
	}
	if ($open && $this->{'CR+LF'} ne "\x0A") {
		$this->{'CR+LF'} = "\x0A";
		$message =~ s/\x0D\x0A/\x0A/g;
	}
}

#直接SMTPを利用する
if (!$open && $this->{'host'}) {
	require CMONOS::CGI_Lib::TcpClient;
	if (TcpClient->new(
				'handle'	=>	\*MAIL,
				'host'		=>	$this->{'host'},
				'portname'	=>	'smtp',
			)->tcp_connect) {
		binmode MAIL;
		$this->{'CR+LF'} = "\x0D\x0A";
		$message =~ s/\x0A/\x0D\x0A/g;
	
		#行頭に"."があるときはもう一つ"."を付ける
		$message =~ s/^\./\.\./g;
	
		if ($this->status != 220) {
			push(@{$this->{'dialog'}},'メールの送信に失敗しました。ホストはSMTPを受け付けません。');
			$this->quit(1);
			$this->close_mail;
			return 0;
		}
		if (!$this->helo) {
			$this->quit(1);
			$this->close_mail;
			return 0;
		}
		if (!$this->mail_from($header{'From'},$header{'Reply-To'})) {
			$this->quit(1);
			$this->close_mail;
			return 0;
		}
		if (!$this->rcpt_to($rcpt_to,$header{'To'},$header{'Cc'},$header{'Bcc'})) {
			$this->quit(1);
			$this->close_mail;
			return 0;
		}
		print MAIL 'DATA', $this->{'CR+LF'};
		if ($this->status != 354) {
			push(@{$this->{'dialog'}},'ホストはメールを受け付けません。');
			$this->quit(1);
			$this->close_mail;
			return 0;
		}
		$open = 2;
		delete $header{'Bcc'};
	}
}

#送信失敗
if (!$open) {
	push(@{$this->{'dialog'}},'送信に失敗しました。');
	return 0;
}

#送信先調整
if (ref $rcpt_to eq 'ARRAY') {
	if (defined $header{'To'}  && $header{'To'} eq 'RCPT TO') {
		$header{'To'} = join(', ',@$rcpt_to);
	} elsif (defined $header{'Cc'} && $header{'Cc'} eq 'RCPT TO') {
		$header{'Cc'} = join(', ',@$rcpt_to);
	} else {
		$header{'Bcc'} .= ', ' if defined $header{'Bcc'};
		$header{'Bcc'} .= join(', ',@$rcpt_to);
	}
}

#送信先が一つの時 To にする
if (!$header{'To'}) {
	if (!$header{'Bcc'} && $header{'Cc'} && $header{'Cc'} !~ /,/) {
		$header{'To'} = $header{'Cc'};
		delete $header{'Cc'};
	} elsif (!$header{'Cc'} && $header{'Bcc'} && $header{'Bcc'} !~ /,/) {
		$header{'To'} = $header{'Bcc'};
		delete $header{'Bcc'};
	}
}

#ヘッダが長くなりすぎないよう改行
$header{'To'} =~ s/[\x09\x0A\x0C\x0D\x20]*,[\x09\x0A\x0C\x0D\x20]*/,$this->{'CR+LF'}      /gos if defined $header{'To'} && $header{'To'} ne '';
$header{'Cc'} =~ s/[\x09\x0A\x0C\x0D\x20]*,[\x09\x0A\x0C\x0D\x20]*/,$this->{'CR+LF'}      /gos if defined $header{'Cc'} && $header{'Cc'} ne '';
$header{'Bcc'} =~ s/[\x09\x0A\x0C\x0D\x20]*,[\x09\x0A\x0C\x0D\x20]*/,$this->{'CR+LF'}      /gos if defined $header{'Bcc'} && $header{'Bcc'} ne '';

#メールヘッダの送信
while (my($k,$v) = each %header) {
	next if !defined $v || $v eq '';
	$v =~ s/"//g;
	print MAIL $k, ': ', $this->header_to_base64($v), $this->{'CR+LF'};
}

#添付ファイルがある場合multipart/mixedヘッダを送信
if (@files) {
	$boundary = '=_CMONOS_' . time . '_MIME_=';
	print MAIL 'Content-Type: multipart/mixed; boundary="', $boundary, '"', $this->{'CR+LF'};
	print MAIL $this->{'CR+LF'};
	print MAIL '> This message is in MIME format. Since your mail reader does not understand', $this->{'CR+LF'};
	print MAIL 'this format, some or all of this message may not be legible.', $this->{'CR+LF'};
	print MAIL $this->{'CR+LF'};
	print MAIL $this->{'CR+LF'};
	print MAIL '--', $boundary, $this->{'CR+LF'};
}

print MAIL 'Content-Transfer-Encoding: 7bit', $this->{'CR+LF'};
print MAIL 'Content-Type: text/plain; charset="ISO-2022-JP"', $this->{'CR+LF'};
print MAIL $this->{'CR+LF'};
print MAIL $message, $this->{'CR+LF'};
print MAIL $this->{'CR+LF'};

#添付ファイルが無い場合送信終了
if (@files) {
	my(%count);
	@files = grep(!$count{$_}++,@files);
	undef %count;

	#ファイルの添付
	FILE:
	foreach my $file (@files) {
		if ($file =~ /^<([^>]+)>$/) {
			my $param = $1;
			NUM:
			foreach my $n ($this->{'input'}->get_posted_file_numbers($param)) {
				my %properties = $this->{'input'}->get_posted_file_properties_from_param($param,$n);
				next NUM unless $properties{'filename'};
				$this->add_file(
					(($properties{'savedir'} && $properties{'savename'}) ? $properties{'savedir'} . '/' . $properties{'savename'} : $this->{'input'}->get_clipboard($param,$n) . '/data'),
					$boundary,
					$properties{'filename'},
					$properties{'type'}
				);
			}
		} else {
			$this->add_file($file,$boundary);
		}
	}
	print MAIL '--', $boundary, '--', $this->{'CR+LF'};
}

print MAIL $this->{'CR+LF'};

#SMTP
if ($open == 2) {
	print MAIL '.', $this->{'CR+LF'};
	$this->quit;
}

$this->close_mail($unlink,@files);
return 1;
}

#ファイル添付
sub add_file ($$$;$$) {
my($this,$path,$boundary,$filename,$filetype) = @_;
local(*ATTACH);
if (ref $path) {
	if (%$path && $path->{'type'} && $path->{'data'}) {
		$filename = $this->header_to_base64($filename || $path->{'name'});
		print MAIL '--', $boundary, $this->{'CR+LF'};
		print MAIL 'Content-Transfer-Encoding: base64', $this->{'CR+LF'};
		print MAIL 'Content-Type: ', $path->{'type'}, '; name="', $filename, '"', $this->{'CR+LF'};
		print MAIL 'Content-Disposition: attachment; filename="', $filename, '"', $this->{'CR+LF'};
		print MAIL $this->{'CR+LF'};
		print MAIL $this->data_to_base64($path->{'data'},1), $this->{'CR+LF'};
		print MAIL $this->{'CR+LF'};
	} else {
		push(@{$this->{'dialog'}},'ファイル添付に失敗しました。指定されたデータが存在しません。');
	}
} else {
	$filename = $path if !$filename;
	$filename = $1 if $filename =~ /([^\/\\:]+)$/;
	if (!$filetype) {
		$filetype = ($filename =~ /\.([^\.]+)$/) ? lc($1) : '';
		$filetype = ($filetype && $SendMail::mime_type->{$filetype}) ? $SendMail::mime_type->{$filetype} : 'application/octet-stream';
	}
	$filename = $this->header_to_base64($filename);
	my $file_size = -s $path;
	if ($this->{'limit'} && $file_size && $file_size > $this->{'limit'}) {
		push(@{$this->{'dialog'}},'ファイル添付に失敗しました。ファイルサイズが' . $this->{'limit'} . 'バイトを超えています。' . $filename);
		return;
	}
	if ($file_size && open(ATTACH, $path)) {
		binmode ATTACH;
		flock(ATTACH, LOCK_SH);
		print MAIL '--', $boundary, $this->{'CR+LF'};
		print MAIL 'Content-Transfer-Encoding: base64', $this->{'CR+LF'};
		print MAIL 'Content-Type: ', $filetype, '; name="', $filename, '"', $this->{'CR+LF'};
		print MAIL 'Content-Disposition: attachment; filename="', $filename, '"', $this->{'CR+LF'};
		print MAIL $this->{'CR+LF'};
		my $buf;
		while (read(ATTACH,$buf,54)) {
			print MAIL $this->data_to_base64($buf), $this->{'CR+LF'};
		}
		print MAIL $this->{'CR+LF'};
		close(ATTACH);
	} else {
		push(@{$this->{'dialog'}},'ファイル添付に失敗しました。ファイルがみつかりません。');
	}
}
}

#応答コード
sub status ($$) {
my $this = shift;
while (<MAIL>) {
	return $1 if $_ =~ /^([0-9]{3})/;
}
}

#sendmail/ソケットを閉じて、不要になった添付ファイルを削除する
sub close_mail ($@) {
my($this,$unlink,@files) = @_;
close (MAIL);

return unless $unlink && @files > 0;

@files = grep($_ !~ /^<.*>$/,@files);
unlink @files;
}

#接続を閉じる
sub quit ($$) {
my($this,$rset) = @_;
print MAIL 'RSET', $this->{'CR+LF'} if defined $rset && $rset;
$this->status;
print MAIL 'QUIT', $this->{'CR+LF'};
return $this->status;
}

#HELOを送信
sub helo ($$) {
my $this = shift;
print MAIL 'HELO ', $this->{'my_host'}, $this->{'CR+LF'};
return 1 if $this->status == 250;
return 0;
}

#MAIL FROMを送信
sub mail_from ($@) {
my($this,@addr) = @_;
my($addr,$status);
@addr = $this->get_address(@addr,$this->{'cgi_master'});
foreach $addr (@addr) {
	print MAIL 'MAIL FROM:<', $addr, '>', $this->{'CR+LF'};
	$status = $this->status;
	last if $status == 250;
}
if ($status != 250) {
	push(@{$this->{'dialog'}},'送信者メールボックスが無効です。' . $addr);
	return 0;
}
return 1;
}

#RCPT TOを送信
sub rcpt_to ($$@) {
my($this,$rcpt_to,@addr) = @_;
my($count,$status,$addr);
my $return = 0;
@addr = $this->get_address(@addr) if @addr;
push(@addr,@$rcpt_to) if ref $rcpt_to eq 'ARRAY';

foreach $addr (@addr) {
	$count++;
	if ($count > 100) {
		push(@{$this->{'dialog'}},'送信先は100件以内にしてください。') if $count == 101;
		push(@{$this->{'bad_address'}},$addr);
		next;
	}
	print MAIL 'RCPT TO:<', $addr, '>', $this->{'CR+LF'};
	$status = $this->status;
	if ($status == 250) {
		$return = 1;
	} else {
		push(@{$this->{'bad_address'}},$addr);
		if ($status == 450) {
			push(@{$this->{'dialog'}},'メールボックスが込み合っています。' . $addr);
		} elsif ($status == 530) {
			push(@{$this->{'dialog'}},'送信を拒否されました。送信ホスト名が正しくありません。' . $addr);
		} elsif ($status == 550) {
			push(@{$this->{'dialog'}},'メールボックスにアクセスできません。' . $addr);
		} elsif ($status == 553) {
			push(@{$this->{'dialog'}},'中継が拒絶されました。' . $addr);
		} else {
			push(@{$this->{'dialog'}},'エラー：' . $status . ' ' . $addr);
		}
	}
}

return $return;
}

#メールアドレスのみ取り出す
sub get_address ($@) {
my($this,@addr) = @_;
my($ad,%addr);
foreach $ad (@addr) {
	next if $ad eq 'RCPT TO' || $ad eq '';
	$addr{$1} = 1 while $ad =~ /([0-9a-zA-Z_\-\+\.]+\@[0-9a-zA-Z_\-]+\.[0-9a-zA-Z_\-\.]+)/gs;
}
return keys %addr;
}

#Dateを取得
sub date ($) {
my $this = shift;
my($sec,$min,$hour,$mday,$mon,$year,$wday,@mon,@wday);
@wday = ('Sun','Mon','Tue','Wed','Thu','Fri','Sat');
@mon = ('Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec');

unless (defined $this->{'TZ'}) {
	if (defined $ENV{'TZ'} && $ENV{'TZ'} =~ /^([a-zA-Z]+)([+\-]+)([0-9]+)([a-zA-Z]*)$/) {
		my ($tz,$z,$t,$s) = ($1,$2,$3,$4);
		$z = ($z eq '+') ? '-' : '+';
		$t = sprintf("%.4d",$t * 100);
		$s = ' ' . $s if defined $s && $s ne '';
		$this->{'TZ'} = ' ' . $z . $t . ' (' . $tz . $s . ')';
	} else {
		$this->{'TZ'} = '';
	}
}

($sec,$min,$hour,$mday,$mon,$year,$wday) = localtime(time);
$sec = sprintf("%.2d",$sec);
$min = sprintf("%.2d",$min);
$hour = sprintf("%.2d",$hour);
$year += 1900;

return "$wday[$wday], $mday $mon[$mon] $year $hour:$min:$sec$this->{'TZ'}";
}

######################################################################
#メールヘッダのBASE64エンコード
sub header_to_base64 ($$) {
my($this,$header) = @_;
$header = $cgi::Jcode->set($this->jcode_fallback($header),'utf8')->jis;
utf8::encode($header) if utf8::is_utf8($header);
$header =~ s/(\x1b\$[\@B]([\x21-\x7e]{2})*(\x1b\([BHJ])*)/$this->text_to_base64($1,1)/eg;
$header =~ s/\x1b\([BHJ]//g;
$header =~ s/\"(=\?ISO\-2022\-JP\?B\?.*?\?=)\"/$1/g;
return $header
}

sub text_to_base64 ($$$) {
my($this,$text,$check) = @_;
$text = $cgi::Jcode->set($this->jcode_fallback($text),'utf8')->jis if !$check;
return '=?ISO-2022-JP?B?' . $this->data_to_base64($text) . '?=';
}

sub jcode_fallback ($$) {
my($this,$text) = @_;
if ($this->{'encoding'} && $this->{'encoding'} ne 'utf8' && $this->{'encoding'} ne 'ascii') {
	$text = $cgi::Jcode->set(\$text,$this->{'encoding'})->utf8;
	$this->{'encoding'} = $this->ename_to_jname($cgi::Jcode->icode) if !$this->{'encoding'};
} else {
	$this->{'encoding'} = 'utf8';
}
$text = $this->h_to_z($text);
$text =~ s/⓪/(0)/sg;
$text =~ s/[①⑴⒈⓵❶㈠㊀➀➊]/(1)/sg;
$text =~ s/[②⑵⒉⓶❷㈡㊁➁➋]/(2)/sg;
$text =~ s/[③⑶⒊⓷❸㈢㊂➂➌]/(3)/sg;
$text =~ s/[④⑷⒋⓸❹㈣㊃➃➍]/(4)/sg;
$text =~ s/[⑤⑸⒌⓹❺㈤㊄➄➎]/(5)/sg;
$text =~ s/[⑥⑹⒍⓺❻㈥㊅➅➏]/(6)/sg;
$text =~ s/[⑦⑺⒎⓻❼㈦㊆➆➐]/(7)/sg;
$text =~ s/[⑧⑻⒏⓼❽㈧㊇➇➑]/(8)/sg;
$text =~ s/[⑨⑼⒐⓽❾㈨㊈➈➒]/(9)/sg;
$text =~ s/[⑩⑽⓾❿㈩㊉➉➓]/(10)/sg;
$text =~ s/[⑪⑾⓫]/(11)/sg;
$text =~ s/[⑫⑿⓬]/(12)/sg;
$text =~ s/[⑬⒀⓭]/(13)/sg;
$text =~ s/[⑭⒁⓮]/(14)/sg;
$text =~ s/[⑮⒂⓯]/(15)/sg;
$text =~ s/[⑯⒃⓰]/(16)/sg;
$text =~ s/[⑰⒄⓱]/(17)/sg;
$text =~ s/[⑱⒅⓲]/(18)/sg;
$text =~ s/[⑲⒆⓳]/(19)/sg;
$text =~ s/[⑳⒇⓴]/(20)/sg;
$text =~ s/[⒜ⓐ]/(a)/sg;
$text =~ s/[⒝ⓑ]/(b)/sg;
$text =~ s/[⒞ⓒ]/(c)/sg;
$text =~ s/[⒟ⓓ]/(d)/sg;
$text =~ s/[⒠ⓔ]/(e)/sg;
$text =~ s/[⒡ⓕ]/(f)/sg;
$text =~ s/[⒢ⓖ]/(g)/sg;
$text =~ s/[⒣ⓗ]/(h)/sg;
$text =~ s/[⒤ⓘ]/(i)/sg;
$text =~ s/[⒥ⓙ]/(j)/sg;
$text =~ s/[⒦ⓚ]/(k)/sg;
$text =~ s/[⒧ⓛ]/(l)/sg;
$text =~ s/[⒨ⓜ]/(m)/sg;
$text =~ s/[⒩ⓝ]/(n)/sg;
$text =~ s/[⒪ⓞ]/(o)/sg;
$text =~ s/[⒫ⓟ]/(p)/sg;
$text =~ s/[⒬ⓠ]/(q)/sg;
$text =~ s/[⒭ⓡ]/(r)/sg;
$text =~ s/[⒮Ⓢ]/(s)/sg;
$text =~ s/[⒯ⓣ]/(t)/sg;
$text =~ s/[⒰ⓤ]/(u)/sg;
$text =~ s/[⒱ⓥ]/(v)/sg;
$text =~ s/[⒲ⓦ]/(w)/sg;
$text =~ s/[⒳ⓧ]/(x)/sg;
$text =~ s/[⒴ⓨ]/(y)/sg;
$text =~ s/[⒵ⓩ]/(z)/sg;
$text =~ s/Ⓐ/(A)/sg;
$text =~ s/Ⓑ/(B)/sg;
$text =~ s/Ⓒ/(C)/sg;
$text =~ s/Ⓓ/(D)/sg;
$text =~ s/Ⓔ/(E)/sg;
$text =~ s/Ⓕ/(F)/sg;
$text =~ s/Ⓖ/(G)/sg;
$text =~ s/Ⓗ/(H)/sg;
$text =~ s/Ⓘ/(I)/sg;
$text =~ s/Ⓙ/(J)/sg;
$text =~ s/Ⓚ/(K)/sg;
$text =~ s/Ⓛ/(L)/sg;
$text =~ s/Ⓜ/(M)/sg;
$text =~ s/Ⓝ/(N)/sg;
$text =~ s/Ⓞ/(O)/sg;
$text =~ s/Ⓟ/(P)/sg;
$text =~ s/Ⓠ/(Q)/sg;
$text =~ s/Ⓡ/(R)/sg;
$text =~ s/Ⓢ/(S)/sg;
$text =~ s/Ⓣ/(T)/sg;
$text =~ s/Ⓤ/(U)/sg;
$text =~ s/Ⓥ/(V)/sg;
$text =~ s/Ⓦ/(W)/sg;
$text =~ s/Ⓧ/(X)/sg;
$text =~ s/Ⓨ/(Y)/sg;
$text =~ s/Ⓩ/(Z)/sg;
$text =~ s/㈹/(代)/sg;
$text =~ s/㈺/(呼)/sg;
$text =~ s/㉀/(祭)/sg;
$text =~ s/㉁/(休)/sg;
$text =~ s/㉂/(自)/sg;
$text =~ s/㉃/(至)/sg;
$text =~ s/㊙/(秘)/sg;
$text =~ s/㊚/(男)/sg;
$text =~ s/㊛/(女)/sg;
$text =~ s/㊜/(適)/sg;
$text =~ s/㊝/(優)/sg;
$text =~ s/㊞/(印)/sg;
$text =~ s/㊟/(注)/sg;
$text =~ s/㊠/(項)/sg;
$text =~ s/㊡/(休)/sg;
$text =~ s/㊢/(写)/sg;
$text =~ s/㊣/(正)/sg;
$text =~ s/㊤/(上)/sg;
$text =~ s/㊥/(中)/sg;
$text =~ s/㊦/(下)/sg;
$text =~ s/㊧/(左)/sg;
$text =~ s/㊨/(右)/sg;
$text =~ s/㊩/(医)/sg;
$text =~ s/㊪/(宗)/sg;
$text =~ s/㊰/(夜)/sg;
$text =~ s/㋐/(ア)/sg;
$text =~ s/㋑/(イ)/sg;
$text =~ s/㋒/(ウ)/sg;
$text =~ s/㋓/(エ)/sg;
$text =~ s/㋔/(オ)/sg;
$text =~ s/㋕/(カ)/sg;
$text =~ s/㋖/(キ)/sg;
$text =~ s/㋗/(ク)/sg;
$text =~ s/㋘/(ケ)/sg;
$text =~ s/㋙/(コ)/sg;
$text =~ s/㋚/(サ)/sg;
$text =~ s/㋛/(シ)/sg;
$text =~ s/㋜/(ス)/sg;
$text =~ s/㋝/(セ)/sg;
$text =~ s/㋞/(ソ)/sg;
$text =~ s/㋟/(タ)/sg;
$text =~ s/㋠/(チ)/sg;
$text =~ s/㋡/(ツ)/sg;
$text =~ s/㋢/(テ)/sg;
$text =~ s/㋣/(ト)/sg;
$text =~ s/㋤/(ナ)/sg;
$text =~ s/㋥/(ニ)/sg;
$text =~ s/㋦/(ヌ)/sg;
$text =~ s/㋧/(ネ)/sg;
$text =~ s/㋨/(ノ)/sg;
$text =~ s/㋩/(ハ)/sg;
$text =~ s/㋪/(ヒ)/sg;
$text =~ s/㋫/(フ)/sg;
$text =~ s/㋬/(ヘ)/sg;
$text =~ s/㋭/(ホ)/sg;
$text =~ s/㋮/(マ)/sg;
$text =~ s/㋯/(ミ)/sg;
$text =~ s/㋰/(ム)/sg;
$text =~ s/㋱/(メ)/sg;
$text =~ s/㋲/(モ)/sg;
$text =~ s/㋳/(ヤ)/sg;
$text =~ s/㋴/(ユ)/sg;
$text =~ s/㋵/(ヨ)/sg;
$text =~ s/㋶/(ラ)/sg;
$text =~ s/㋷/(リ)/sg;
$text =~ s/㋸/(ル)/sg;
$text =~ s/㋹/(レ)/sg;
$text =~ s/㋺/(ロ)/sg;
$text =~ s/㋻/(ワ)/sg;
$text =~ s/㋼/(ヰ)/sg;
$text =~ s/㋾/(ヲ)/sg;
$text =~ s/[㈪㊊]/(月)/sg;
$text =~ s/[㈫㊋]/(火)/sg;
$text =~ s/[㈬㊌]/(水)/sg;
$text =~ s/[㈭㊍]/(木)/sg;
$text =~ s/[㈮㊎]/(金)/sg;
$text =~ s/[㈯㊏]/(土)/sg;
$text =~ s/[㈰㊐]/(日)/sg;
$text =~ s/[㈱㊑]/(株)/sg;
$text =~ s/[㈲㊒]/(有)/sg;
$text =~ s/[㈳㊓]/(社)/sg;
$text =~ s/[㈴㊔]/(名)/sg;
$text =~ s/[㈵㊕]/(特)/sg;
$text =~ s/[㈶㊖]/(財)/sg;
$text =~ s/[㈷㊗]/(祝)/sg;
$text =~ s/[㈸㊘]/(労)/sg;
$text =~ s/[㈻㊫]/(学)/sg;
$text =~ s/[㈼㊬]/(監)/sg;
$text =~ s/[㈽㊭]/(企)/sg;
$text =~ s/[㈾㊮]/(資)/sg;
$text =~ s/[㈿㊯]/(協)/sg;
$text =~ s/[～∼]/〜/sg;
$text =~ s/[－⁻−﹣]/-/sg;
return $text;
}

######################################################################
#BASE64エンコード
sub data_to_base64 ($$;$) {
my($this,$data,$return) = @_;
my(@charset) = ('A'..'Z','a'..'z','0'..'9','+','/');
utf8::encode($data) if utf8::is_utf8($data);
$data = unpack("B*", $data);
$data =~ s!(.{1,6})!$charset[ord(pack("B*", '00' . $1))]!g;
$data .= '=' x (3 - (bytes::length($1) / 2));
if ($return) {
	$data =~ s/(.{72})/$1$this->{'CR+LF'}/gs;
	$data =~ s/$this->{'CR+LF'}\Z//s;
}
return $data;
}

########################################################################
#エラーメッセージ
sub dialog ($) {
my $this = shift;
my @dialog = (ref $this->{'dialog'} eq 'ARRAY') ? @{$this->{'dialog'}} : ();
undef @{$this->{'dialog'}};
return @dialog;
}

########################################################################
#不正なアドレス
sub bad_address ($) {
my $this = shift;
my @bad_address = (ref $this->{'bad_address'} eq 'ARRAY') ? @{$this->{'bad_address'}} : ();
undef @{$this->{'bad_address'}};
return @bad_address;
}

########################################################################
#以下はCommon.pmの一部をSendMail.pm用に単純化したもの
########################################################################
#URLを求める
sub my_url ($) {
my $this = shift;
return (($env::ssl || ((!defined $env::ssl || $env::ssl eq '') && ($ENV{'HTTPS'} || ($ENV{'SERVER_PORT'} && $ENV{'SERVER_PORT'} == 443)))) ? 'https://' : 'http://') . ($env::server_name || $this->to_host($ENV{'SERVER_NAME'})) . (($ENV{'SERVER_PORT'} != 80 && $ENV{'SERVER_PORT'} != 443) ? ':' . $ENV{'SERVER_PORT'} : '') . $ENV{'SCRIPT_NAME'};
}

########################################################################
#IPアドレス->HOSTネーム
sub to_host ($$) {
my($this,$addr) = @_;
my($host_name);
return $addr if $addr =~ /[^0-9\.]/;
$host_name = gethostbyaddr(pack("C4", split(/\./, $addr)), 2);
return $addr if !defined $host_name || $host_name eq '';
return $host_name;
}

########################################################################
#ドメイン名取得
sub my_domain ($) {
my $this = shift;
my $mydomain = $env::server_name || $this->to_host($ENV{'SERVER_NAME'});
return $mydomain
}

########################################################################
#ホームディレクトリまでのサーバ内パス･URLパスを求める
sub home_path ($) {
my($mypath,$d,@dir,$document_root,$user_directory);
my $this = shift;
if ($ENV{'SCRIPT_FILENAME'}) {
	$document_root = $ENV{'SCRIPT_FILENAME'};
	$document_root =~ s!/[^/]*$!!;
} else {
	chdir;
	use Cwd;
	$document_root = cwd() || getcwd;
}
$mypath = $ENV{'SCRIPT_NAME'};
$mypath =~ s!/[^/]*$!!;
@dir = split('/',$mypath);
while ($d = pop(@dir)) {
	last unless $document_root =~ s!/$d$!!;
}
push(@dir,$d);
$user_directory = join('/',@dir);
return ($document_root,$user_directory);
}

########################################################################
#URLパスをサーバ内パスに変換する
sub url_to_path ($$) {
my($this,$path) = @_;
return $path if $path !~ m!^/!;

my($document_root,$user_directory) = $this->home_path;
return $path if $path =~ /^$document_root/;
return $path if $document_root =~ /^$path/;

$path =~ s/^$user_directory/$document_root/;
return $path;
}

########################################################################
#桁揃え（72文字に揃える）
sub mfold ($$) {
my($this,$text) = @_;
my $wc = 0;
my $r = 0;
$text =~ s/((http|ftp)s?:\/\/[0-9a-zA-Z_\-\%\.\?#\/\&;=\+]+|[0-9a-zA-Z_\-\+\.]+\@[0-9a-zA-Z_\-]+\.[0-9a-zA-Z_\-\.]+|&[0-9a-zA-Z_#]+;|\x0D\x0A|\x0A|\x0D|.)/$this->split_string($1,\$wc,\$r)/egs;
return $text;
}

sub split_string ($$$$) {
my($this,$word,$wc,$r) = @_;
if ($word =~ /[\x0D\x0A]+/s) {
	$$wc = 0;
	if ($$r) {
		$$r = 0;
		return '';
	} else {
		return $this->{'CR+LF'};
	}
}
my $plus = ($word =~ /^(?:http|ftp|[0-9a-zA-Z_\-\+\.]+\@)/) ? length $word : ($word =~ /^&[#0-9a-zA-Z_]+;$/) ? 1 : (bytes::length($word) >= 2) ? 2 : 1;
$$wc += $plus;
$$r = 0;
if ($$wc == 72) {
	$word .= $this->{'CR+LF'};
	$$wc = 0;
	$$r = 1;
} elsif ($$wc > 72) {
	$word = (($$wc) ? $this->{'CR+LF'} : '') . $word;
	$$wc = $plus;
}
return $word;
}

sub h_to_z ($$) {
my($this,$text) = @_;
return '' if !defined $text || $text eq '';
utf8::decode($text) if !utf8::is_utf8($text);
$text =~ s/･/・/gs;
$text =~ s/ｦ/ヲ/gs;
$text =~ s/ｧ/ァ/gs;
$text =~ s/ｨ/ィ/gs;
$text =~ s/ｩ/ゥ/gs;
$text =~ s/ｪ/ェ/gs;
$text =~ s/ｫ/ォ/gs;
$text =~ s/ｬ/ャ/gs;
$text =~ s/ｭ/ュ/gs;
$text =~ s/ｮ/ョ/gs;
$text =~ s/ｯ/ッ/gs;
$text =~ s/ｰ/ー/gs;
$text =~ s/ｱ/ア/gs;
$text =~ s/ｲ/イ/gs;
$text =~ s/ｳﾞ/ヴ/gs;
$text =~ s/ｳ/ウ/gs;
$text =~ s/ｴ/エ/gs;
$text =~ s/ｵ/オ/gs;
$text =~ s/ｶﾞ/ガ/gs;
$text =~ s/ｶ/カ/gs;
$text =~ s/ｷﾞ/ギ/gs;
$text =~ s/ｷ/キ/gs;
$text =~ s/ｸﾞ/グ/gs;
$text =~ s/ｸ/ク/gs;
$text =~ s/ｹﾞ/ゲ/gs;
$text =~ s/ｹ/ケ/gs;
$text =~ s/ｺﾞ/ゴ/gs;
$text =~ s/ｺ/コ/gs;
$text =~ s/ｻﾞ/ザ/gs;
$text =~ s/ｻ/サ/gs;
$text =~ s/ｼﾞ/ジ/gs;
$text =~ s/ｼ/シ/gs;
$text =~ s/ｽﾞ/ズ/gs;
$text =~ s/ｽ/ス/gs;
$text =~ s/ｾﾞ/ゼ/gs;
$text =~ s/ｾ/セ/gs;
$text =~ s/ｿﾞ/ゾ/gs;
$text =~ s/ｿ/ソ/gs;
$text =~ s/ﾀﾞ/ダ/gs;
$text =~ s/ﾀ/タ/gs;
$text =~ s/ﾁﾞ/ヂ/gs;
$text =~ s/ﾁ/チ/gs;
$text =~ s/ﾂﾞ/ヅ/gs;
$text =~ s/ﾂ/ツ/gs;
$text =~ s/ﾃﾞ/デ/gs;
$text =~ s/ﾃ/テ/gs;
$text =~ s/ﾄﾞ/ド/gs;
$text =~ s/ﾄ/ト/gs;
$text =~ s/ﾅ/ナ/gs;
$text =~ s/ﾆ/ニ/gs;
$text =~ s/ﾇ/ヌ/gs;
$text =~ s/ﾈ/ネ/gs;
$text =~ s/ﾉ/ノ/gs;
$text =~ s/ﾊﾞ/バ/gs;
$text =~ s/ﾊﾟ/パ/gs;
$text =~ s/ﾊ/ハ/gs;
$text =~ s/ﾋﾞ/ビ/gs;
$text =~ s/ﾋﾟ/ピ/gs;
$text =~ s/ﾋ/ヒ/gs;
$text =~ s/ﾌﾞ/ブ/gs;
$text =~ s/ﾌﾟ/プ/gs;
$text =~ s/ﾌ/フ/gs;
$text =~ s/ﾍﾞ/ベ/gs;
$text =~ s/ﾍﾟ/ペ/gs;
$text =~ s/ﾍ/ヘ/gs;
$text =~ s/ﾎﾞ/ボ/gs;
$text =~ s/ﾎﾟ/ポ/gs;
$text =~ s/ﾎ/ホ/gs;
$text =~ s/ﾏ/マ/gs;
$text =~ s/ﾐ/ミ/gs;
$text =~ s/ﾑ/ム/gs;
$text =~ s/ﾒ/メ/gs;
$text =~ s/ﾓ/モ/gs;
$text =~ s/ﾔ/ヤ/gs;
$text =~ s/ﾕ/ユ/gs;
$text =~ s/ﾖ/ヨ/gs;
$text =~ s/ﾗ/ラ/gs;
$text =~ s/ﾘ/リ/gs;
$text =~ s/ﾙ/ル/gs;
$text =~ s/ﾚ/レ/gs;
$text =~ s/ﾛ/ロ/gs;
$text =~ s/ﾜ/ワ/gs;
$text =~ s/ﾝ/ン/gs;
return $text;
}

sub ename_to_jname ($$) {
my($this,$ename) = @_;
return 'utf8' if !$ename;
$ename = lc $ename;
my %ename2j = (
	'shiftjis'		=>	'sjis',
	'x-sjis'		=>	'sjis',
	'shift_jis'		=>	'sjis',
	'euc-jp'		=>	'euc',
	'x-euc-jp'		=>	'euc',
	'7bit-jis'		=>	'jis',
	'x-jis'			=>	'jis',
	'iso-2022-jp'	=>	'jis',
	'utf-16be'		=>	'ucs2',
	'utf-8'			=>	'utf8',
);
if ($ename2j{$ename}) {
	return $ename2j{$ename};
} else {
	$ename =~ tr/-/_/;
	return $ename;
}
}

########################################################################
# メールアドレス書式チェック
sub is_invalid_email_address ($$) {
my($this,$address) = @_;
return 0 if $address =~ m!^[0-9a-zA-Z_\-\+\.]+\@[0-9a-zA-Z_\-]+\.[0-9a-zA-Z_\-\.]+$!i;
return 1;
}

sub is_invalid_email_address_header ($@) {
my($this,$address) = @_;
my(@addr,$addr);
@addr = split(/,[\x09\x0A\x0C\x0D\x20]*/,$address);

foreach $addr (@addr) {
	if ($addr =~ /<([^<>]+)>/) {
		return 1 if $this->is_invalid_email_address($1);
	} else {
		$addr =~ s/\(.*\)//g;
		$addr =~ s/[\x09\x0A\x0C\x0D\x20]+//g;
		return 1 if $this->is_invalid_email_address($addr);
	}
}

return 0;
}



1;